<tr class="noCover"><td class="line"><a name="1">1</a></td><td class="hits"></td><td class="code">&#34;&#34;&#34;Functions to help tracing the application, i.e. via logging.</td></tr><tr class="noCover"><td class="line"><a name="2">2</a></td><td class="hits"></td><td class="code">&#34;&#34;&#34;</td></tr><tr class="coverFull" data-html-tooltip="Covered at least once"><td class="line"><a name="3">3</a></td><td class="hits">1</td><td class="code">import logging</td></tr><tr class="coverFull" data-html-tooltip="Covered at least once"><td class="line"><a name="4">4</a></td><td class="hits">1</td><td class="code">import multiprocessing</td></tr><tr class="noCover"><td class="line"><a name="5">5</a></td><td class="hits"></td><td class="code"></td></tr><tr class="coverFull" data-html-tooltip="Covered at least once"><td class="line"><a name="6">6</a></td><td class="hits">1</td><td class="code">import humanize</td></tr><tr class="noCover"><td class="line"><a name="7">7</a></td><td class="hits"></td><td class="code"></td></tr><tr class="coverFull" data-html-tooltip="Covered at least once"><td class="line"><a name="8">8</a></td><td class="hits">1</td><td class="code">logger &#61; logging.getLogger(__name__)</td></tr><tr class="noCover"><td class="line"><a name="9">9</a></td><td class="hits"></td><td class="code"></td></tr><tr class="noCover"><td class="line"><a name="10">10</a></td><td class="hits"></td><td class="code"></td></tr><tr class="coverFull" data-html-tooltip="Covered at least once"><td class="line"><a name="11">11</a></td><td class="hits">1</td><td class="code">class DownloadTracer:</td></tr><tr class="noCover"><td class="line"><a name="12">12</a></td><td class="hits"></td><td class="code">    &#34;&#34;&#34;Log meta-info about any long-running download task.</td></tr><tr class="noCover"><td class="line"><a name="13">13</a></td><td class="hits"></td><td class="code"></td></tr><tr class="noCover"><td class="line"><a name="14">14</a></td><td class="hits"></td><td class="code">    Note: This is Thread-safe -- can be used with &#34;&#96;multiprocessing.dummy&#96;&#34; package.</td></tr><tr class="noCover"><td class="line"><a name="15">15</a></td><td class="hits"></td><td class="code">        Downloads are I/O bound anyhow, so multithreading works well. </td></tr><tr class="noCover"><td class="line"><a name="16">16</a></td><td class="hits"></td><td class="code"></td></tr><tr class="noCover"><td class="line"><a name="17">17</a></td><td class="hits"></td><td class="code">    WARNING: This is _not_ concurrency-safe. Do not use this with &#34;&#96;multiprocessing.Pool&#96;&#34; directly.</td></tr><tr class="noCover"><td class="line"><a name="18">18</a></td><td class="hits"></td><td class="code">    &#34;&#34;&#34;</td></tr><tr class="coverFull" data-html-tooltip="Covered at least once"><td class="line"><a name="19">19</a></td><td class="hits">1</td><td class="code">    def __init__(self, service_name: str, log_every_count_bytes:int &#61; 10000):</td></tr><tr class="coverFull" data-html-tooltip="Covered at least once"><td class="line"><a name="20">20</a></td><td class="hits">1</td><td class="code">        self._name &#61; service_name</td></tr><tr class="coverFull" data-html-tooltip="Covered at least once"><td class="line"><a name="21">21</a></td><td class="hits">1</td><td class="code">        self._threshold &#61; log_every_count_bytes</td></tr><tr class="coverFull" data-html-tooltip="Covered at least once"><td class="line"><a name="22">22</a></td><td class="hits">1</td><td class="code">        self._total_bytes_downloaded &#61; 0</td></tr><tr class="coverFull" data-html-tooltip="Covered at least once"><td class="line"><a name="23">23</a></td><td class="hits">1</td><td class="code">        self._prior_logged_total_bytes_downloaded &#61; 0</td></tr><tr class="coverFull" data-html-tooltip="Covered at least once"><td class="line"><a name="24">24</a></td><td class="hits">1</td><td class="code">        self._lock &#61; multiprocessing.Lock()</td></tr><tr class="noCover"><td class="line"><a name="25">25</a></td><td class="hits"></td><td class="code"></td></tr><tr class="coverFull" data-html-tooltip="Covered at least once"><td class="line"><a name="26">26</a></td><td class="hits">1</td><td class="code">    &#64;property</td></tr><tr class="coverFull" data-html-tooltip="Covered at least once"><td class="line"><a name="27">27</a></td><td class="hits">1</td><td class="code">    def total_bytes(self):</td></tr><tr class="coverNone" data-html-tooltip="Not covered"><td class="line"><a name="28">28</a></td><td class="hits">0</td><td class="code">        return self._total_bytes_downloaded</td></tr><tr class="noCover"><td class="line"><a name="29">29</a></td><td class="hits"></td><td class="code"></td></tr><tr class="coverFull" data-html-tooltip="Covered at least once"><td class="line"><a name="30">30</a></td><td class="hits">1</td><td class="code">    def trace(self, data_len: int):</td></tr><tr class="noCover"><td class="line"><a name="31">31</a></td><td class="hits"></td><td class="code">        &#34;&#34;&#34;Add data_len to the running total.</td></tr><tr class="noCover"><td class="line"><a name="32">32</a></td><td class="hits"></td><td class="code">        &#34;&#34;&#34;</td></tr><tr class="coverFull" data-html-tooltip="Covered at least once"><td class="line"><a name="33">33</a></td><td class="hits">1</td><td class="code">        self._lock.acquire()</td></tr><tr class="coverFull" data-html-tooltip="Covered at least once"><td class="line"><a name="34">34</a></td><td class="hits">1</td><td class="code">        try:</td></tr><tr class="coverFull" data-html-tooltip="Covered at least once"><td class="line"><a name="35">35</a></td><td class="hits">1</td><td class="code">            self._total_bytes_downloaded &#43;&#61; data_len</td></tr><tr class="coverFull" data-html-tooltip="Covered at least once"><td class="line"><a name="36">36</a></td><td class="hits">1</td><td class="code">            if self._total_bytes_downloaded &gt; self._threshold &#43; self._prior_logged_total_bytes_downloaded:</td></tr><tr class="coverFull" data-html-tooltip="Covered at least once"><td class="line"><a name="37">37</a></td><td class="hits">1</td><td class="code">                logger.info(f&#39;Total downloaded from {self._name}: {humanize.naturalsize(self._total_bytes_downloaded)}&#39;)</td></tr><tr class="coverFull" data-html-tooltip="Covered at least once"><td class="line"><a name="38">38</a></td><td class="hits">1</td><td class="code">                self._prior_logged_total_bytes_downloaded &#61; self._total_bytes_downloaded</td></tr><tr class="noCover"><td class="line"><a name="39">39</a></td><td class="hits"></td><td class="code">        finally:</td></tr><tr class="coverFull" data-html-tooltip="Covered at least once"><td class="line"><a name="40">40</a></td><td class="hits">1</td><td class="code">            self._lock.release()</td></tr><tr class="noCover"><td class="line"><a name="41">41</a></td><td class="hits"></td><td class="code"></td></tr><tr class="noCover"><td class="line"><a name="42">42</a></td><td class="hits"></td><td class="code"></td></tr><tr class="coverFull" data-html-tooltip="Covered at least once"><td class="line"><a name="43">43</a></td><td class="hits">1</td><td class="code">one_kilobyte &#61; 1000</td></tr><tr class="coverFull" data-html-tooltip="Covered at least once"><td class="line"><a name="44">44</a></td><td class="hits">1</td><td class="code">one_megabyte &#61; 1000 * one_kilobyte</td></tr><tr class="noCover"><td class="line"><a name="45">45</a></td><td class="hits"></td><td class="code"></td></tr><tr class="coverFull" data-html-tooltip="Covered at least once"><td class="line"><a name="46">46</a></td><td class="hits">1</td><td class="code">netdot_download_tracer &#61; DownloadTracer(&#39;Netdot&#39;, one_megabyte)</td></tr><tr class="noCover"><td class="line"><a name="47">47</a></td><td class="hits"></td><td class="code"># alias to enable trace.downloads(response)</td></tr><tr class="coverFull" data-html-tooltip="Covered at least once"><td class="line"><a name="48">48</a></td><td class="hits">1</td><td class="code">netdot_downloads &#61; netdot_download_tracer.trace</td></tr>